from .gameBoards import generateBoard


class Ship:
    def __init__(self, size, shipAbbr, shipName):
        """Inits SampleClass with blah."""
        self.operational = True
        self.size = size
        self.hitsRemaining = size
        self.placement = []
        self.shipAbbr = shipAbbr
        self.shipName = shipName


class Player:
    def __init__(self, computer=False, computerLevel=0):
        self.name = None
        self.computer = computer
        self.computerLevel = computerLevel
        self.huntMode = False
        self.huntModeStartShot = 0
        self.lastShotResult = 'M'
        self.huntBoard = generateBoard()
        self.shipCount = 5
        self.moveCount = 0
        self.masterPlacement = []
        self.masterBoard = {}
        self.carrier = Ship(5, 'C', 'Carrier')
        self.battleship = Ship(4, 'B', 'Battleship')
        self.cruiser = Ship(3, 'c', 'Cruiser')
        self.submarine = Ship(3, 'S', 'Submarine')
        self.destroyer = Ship(2, 'D', 'Destroyer')
        self.xAxis = [i for i in range(1, 11)]
        self.yAxis = ['A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J']
        self.carrierProb = 0
        self.battleshipProb = 0
        self.cruiserProb = 0
        self.submarineProb = 0
        self.destroyerProb = 0
        self.cumlativeProb = 0

    def reduceShipCount(self):
        self.shipCount -= 1

    def reduceCarrierCount(self):
        self.carrier.hitsRemaining -= 1

    def reduceBattleshipCount(self):
        self.battleship.hitsRemaining -= 1

    def reduceCruiserCount(self):
        self.cruiser.hitsRemaining -= 1

    def reduceSubmarineCount(self):
        self.submarine.hitsRemaining -= 1

    def reduceDestroyerCount(self):
        self.destroyer.hitsRemaining -= 1

    def setHuntMode(self, setting):
        self.huntMode = setting

    def placeCarrier(self, positionList):
        self.carrier.placement = positionList
        self.masterPlacement.extend(positionList)

    def placeBattleship(self, positionList):
        self.battleship.placement = positionList
        self.masterPlacement.extend(positionList)

    def placeCruiser(self, positionList):
        self.cruiser.placement = positionList
        self.masterPlacement.extend(positionList)

    def placeSubmarine(self, positionList):
        self.submarine.placement = positionList
        self.masterPlacement.extend(positionList)

    def placeDestroyer(self, positionList):
        self.destroyer.placement = positionList
        self.masterPlacement.extend(positionList)

    def removeCarrier(self):
        for i in self.carrier.placement:
            self.masterBoard[i] = '0'
            if i in self.masterPlacement:
                self.masterPlacement.remove(i)
            self.carrier.placement = []

    def removeBattleship(self):
        for i in self.battleship.placement:
            self.masterBoard[i] = '0'
            if i in self.masterPlacement:
                self.masterPlacement.remove(i)
            self.battleship.placement = []

    def removeCruiser(self):
        for i in self.cruiser.placement:
            self.masterBoard[i] = '0'
            if i in self.masterPlacement:
                self.masterPlacement.remove(i)
            self.cruiser.placement = []

    def removeSubmarine(self):
        for i in self.submarine.placement:
            self.masterBoard[i] = '0'
            if i in self.masterPlacement:
                self.masterPlacement.remove(i)
            self.submarine.placement = []

    def removeDestroyer(self):
        for i in self.destroyer.placement:
            self.masterBoard[i] = '0'
            if i in self.masterPlacement:
                self.masterPlacement.remove(i)
            self.destroyer.placement = []

    def checkShotMaster(self, shot):
        if shot in self.masterPlacement:
            return True
        else:
            return False

    def checkCarrierShot(self, shot):
        if shot in self.carrier.placement:
            return True
        else:
            return False

    def checkBattleshipShot(self, shot):
        if shot in self.battleship.placement:
            return True
        else:
            return False

    def checkCruiserShot(self, shot):
        if shot in self.cruiser.placement:
            return True
        else:
            return False

    def checkSubmarineShot(self, shot):
        if shot in self.submarine.placement:
            return True
        else:
            return False

    def checkDestroyerShot(self, shot):
        if shot in self.destroyer.placement:
            return True
        else:
            return False

    def checkForMiss(self, currentList):
        for i in currentList:
            if self.huntBoard[i] == 'M':
                return True
        return False

    def createHorizonalList(self):
        horizontalList = []
        for y in self.yAxis:
            newList = []
            for x in self.xAxis:
                pos = f'{y}:{x}'
                newList.append(pos)
            horizontalList.append(newList)
        return horizontalList

    def createVerticalList(self):
        verticalList = []
        for x in self.xAxis:
            newList = []
            for y in self.yAxis:
                pos = f'{y}:{x}'
                newList.append(pos)
            verticalList.append(newList)
        return verticalList

    def updateHorizontalProb(self, shipSize):
        horizontalCount = generateBoard()
        horizontalList = self.createHorizonalList()
        for i in horizontalList:
            newStart = 0
            newStop = shipSize
            for j in range(len(i)):
                if j + shipSize <= len(i):
                    tempList = (i[newStart:newStop])
                    newStart += 1
                    newStop += 1
                    if self.checkForMiss(tempList) is not True:
                        tempCount = 1
                    else:
                        tempCount = 0

                    for k in tempList:
                        tempX = int(horizontalCount[k])
                        tempX = tempX + tempCount
                        horizontalCount[k] = tempX

        return horizontalCount

    def updateVerticalProb(self, shipSize):
        verticalCount = generateBoard()
        verticalList = self.createVerticalList()
        for i in verticalList:
            newStart = 0
            newStop = shipSize
            for j in range(len(i)):
                if j + shipSize <= len(i):
                    tempList = (i[newStart:newStop])
                    newStart += 1
                    newStop += 1
                    if self.checkForMiss(tempList) is not True:
                        tempCount = 1
                    else:
                        tempCount = 0

                    for k in tempList:
                        tempX = int(verticalCount[k])
                        tempX = tempX + tempCount
                        verticalCount[k] = tempX

        return verticalCount

    def updateTotalProb(self, shipSize):
        totalProb = {}
        horizontalProb = self.updateHorizontalProb(shipSize)
        verticalProb = self.updateVerticalProb(shipSize)
        for y in self.yAxis:
            for x in self.xAxis:
                pos = f'{y}:{x}'
                xCount = horizontalProb[pos]
                yCount = verticalProb[pos]
                totalProb[pos] = (xCount + yCount)
        return totalProb

    def updateCumlativeProb(self):
        tempcumlativeProb = {}
        self.carrierProb = self.updateTotalProb(self.carrier.size)
        self.battleshipProb = self.updateTotalProb(self.battleship.size)
        self.cruiserProb = self.updateTotalProb(self.cruiser.size)
        self.submarineProb = self.updateTotalProb(self.submarine.size)
        self.destroyerProb = self.updateTotalProb(self.destroyer.size)
        for y in self.yAxis:
            for x in self.xAxis:
                pos = f'{y}:{x}'
                carrierCount = self.carrierProb[pos]
                battleshipCount = self.battleshipProb[pos]
                cruiserCount = self.cruiserProb[pos]
                submarineCount = self.submarineProb[pos]
                destroyerCount = self.destroyerProb[pos]
                tempCount = (carrierCount +
                             battleshipCount +
                             cruiserCount +
                             submarineCount +
                             destroyerCount)
                tempcumlativeProb[pos] = tempCount
        self.cumlativeProb = tempcumlativeProb

    def updateMasterBoard(self):
        self.masterBoard = generateBoard()
        for i in self.carrier.placement:
            self.masterBoard[i] = 'C'
        for i in self.battleship.placement:
            self.masterBoard[i] = 'B'
        for i in self.cruiser.placement:
            self.masterBoard[i] = 'c'
        for i in self.submarine.placement:
            self.masterBoard[i] = 'S'
        for i in self.destroyer.placement:
            self.masterBoard[i] = 'D'



from .gameModules.players import Player
from .gameModules.autoShipPlacement import placeAllships
from .gameModules.playerShipPlacement import playerPlacement
from .gameModules.playerNumberFunctions import ComputerGameFunction
from .gameModules.playerNumberFunctions import OnePlayerGameFunction


def playerCountCheck(prompt):
    while True:
        try:
            playerCount = int(input(prompt))

        except ValueError:
            print("Sorry, please enter either a 0, 1, or 2.")
            continue

        if playerCount not in [0, 1, 2]:
            print('Sorry, your responce must be 0, 1, or 2')
            continue

        else:
            break
    return playerCount


def computerLevelCheck(prompt):
    while True:
        try:
            compLevel = int(input(prompt))
        except ValueError:
            print("Sorry, please enter a number between 0 and 4.")
            continue
        if compLevel not in [0, 1, 2, 3, 4]:
            print('Sorry, your responce must be between 0 and 4')
            continue
        else:
            break
    return compLevel


def newComputerGame():
    player1 = Player()
    player1.computerLevel = computerLevelCheck('Please enter the level of '
                                               'computer player 1 (0 is the '
                                               'lowest 4 is the highest): ')
    player1.computer = True
    player2 = Player()
    player2.computerLevel = computerLevelCheck('Please enter the level of '
                                               'computer player 2 (0 is the '
                                               'lowest 4 is the highest): ')
    player2.computer = True
    placeAllships(player1)
    placeAllships(player2)
    player1.name = 'Player 1'
    player2.name = 'Player 2'
    gl = ComputerGameFunction(player1, player2)
    return gl[-1]


def newOnePlayerGame():
    player1 = Player()
    player1.name = input('Please enter the name of Player 1: ')
    player1.computer = False
    playerPlacement(player1)
    player2 = Player()
    player2.computerLevel = computerLevelCheck('Please enter the level of '
                                               'computer player 2 (1 is the '
                                               'lowest 4 is the highest) ')
    player2.computer = True
    player2.name = 'Computron'
    placeAllships(player2)
    gl = OnePlayerGameFunction(player1, player2)
    return gl[-1]


def main():
    print('Welcome to Battleship')
    playerNumbers = playerCountCheck('Please enter the number of players: ')
    if playerNumbers == 0:
        gl = newComputerGame()
        print(f'{gl.get("gameWinner", " ")} won the game '
              f'in {gl.get("offPlayerMoves", " ")} moves')

    elif playerNumbers == 1:
        gl = newOnePlayerGame()
        print(f'{gl.get("gameWinner", " ")} won the '
              f'game in {gl.get("offPlayerMoves", " ")} moves')

        print('1 Player Game')
    elif playerNumbers == 2:
        # TODO 2 Player Function
        print('2 Player Game')


if __name__ == "__main__":
    main()



import random


def fullTurn(shot, offPlayer, defPlayer):
    offPlayer.moveCount += 1
    ship = checkShot(shot, defPlayer)
    if ship == 'Miss':
        return {'shot': shot,
                'shotResult': 'M',
                'gameStatus': 'in-progress',
                'offPlayer': offPlayer.name,
                'defPlayer': defPlayer.name,
                'offPlayerMoves': offPlayer.moveCount}
    else:
        reduceShipHits(ship, defPlayer)
        shipHitsTotal = checkShipHits(ship, defPlayer)
        if shipHitsTotal >= 1:
            if offPlayer.huntMode is False:
                offPlayer.huntMode = True
                offPlayer.huntModeStartShot = shot
            return {'shot': shot,
                    'shotResult': 'H',
                    'gameStatus': 'in-progress',
                    'offPlayer': offPlayer.name,
                    'defPlayer': defPlayer.name,
                    'offPlayerMoves': offPlayer.moveCount}
        else:
            defPlayer.shipCount -= 1
            offPlayer.huntMode = False
            if defPlayer.shipCount >= 1:
                return {'shot': shot, 'shotResult': 'H',
                        'shipSunk': ship,
                        'gameStatus': 'in-progress',
                        'offPlayer': offPlayer.name,
                        'defPlayer': defPlayer.name,
                        'offPlayerMoves': offPlayer.moveCount}
            else:
                return {'shot': shot,
                        'shotResult': 'H',
                        'gameStatus': 'over',
                        'gameWinner': offPlayer.name,
                        'offPlayer': offPlayer.name,
                        'defPlayer': defPlayer.name,
                        'offPlayerMoves': offPlayer.moveCount}


def checkShot(shot, defPlayer):
    if defPlayer.checkShotMaster(shot) is True:
        if defPlayer.checkCarrierShot(shot) is True:
            return "Carrier"
        elif defPlayer.checkBattleshipShot(shot) is True:
            return "Battleship"
        elif defPlayer.checkCruiserShot(shot) is True:
            return "Cruiser"
        elif defPlayer.checkSubmarineShot(shot) is True:
            return "Submarine"
        elif defPlayer.checkDestroyerShot(shot) is True:
            return "Destroyer"
    else:
        return 'Miss'


def reduceShipHits(ship, defPlayer):
    if ship == 'Carrier':
        defPlayer.reduceCarrierCount()
    elif ship == 'Battleship':
        defPlayer.reduceBattleshipCount()
    elif ship == 'Cruiser':
        defPlayer.reduceCruiserCount()
    elif ship == 'Submarine':
        defPlayer.reduceSubmarineCount()
    elif ship == 'Destroyer':
        defPlayer.reduceDestroyerCount()


def checkShipHits(ship, defPlayer):
    return getattr(getattr(defPlayer, ship.lower()), 'hitsRemaining')


def randomPlayer(player1, player2):
    offPlayer = random.choice([player1, player2])
    if offPlayer == player1:
        defPlayer = player2
    else:
        defPlayer = player1
    return(offPlayer, defPlayer)



import random


def computerShot(offPlayer):

    if offPlayer.computerLevel == 0:
        # Random shot
        shot = randomComputerShot(offPlayer)

    elif offPlayer.computerLevel == 1:
        shot = randomComputerShotHuntMode(offPlayer)

    elif offPlayer.computerLevel == 2:
        shot = parityComputerShotHuntMode(offPlayer)

    elif offPlayer.computerLevel == 3:
        offPlayer.updateCumlativeProb()
        shot = probDensityComputerShot(offPlayer)

    elif offPlayer.computerLevel == 4:
        offPlayer.updateCumlativeProb()
        shot = ProbDensityComputerShotHuntMode(offPlayer)
    return shot


def randomComputerShot(offPlayer):

    while True:
        pshot = random.choice(list(offPlayer.huntBoard.keys()))
        if offPlayer.huntBoard[pshot] != '0':
            pass
        else:
            return pshot


def randomComputerShotHuntMode(offPlayer):
    if offPlayer.huntMode is True:
        return(huntShotReturn(offPlayer))
    else:
        shot = randomComputerShot(offPlayer)
        return shot


def parityComputerShotHuntMode(offPlayer):
    if offPlayer.huntMode is True:
        return(huntShotReturn(offPlayer))
    else:
        shot = paritySearch(offPlayer)
        return shot


def ProbDensityComputerShotHuntMode(offPlayer):
    if offPlayer.huntMode is True:
        return(huntShotReturn(offPlayer))
    else:
        shot = probDensityComputerShot(offPlayer)
        return shot


def paritySearch(offPlayer):
    keyList = []
    xAxis = [i for i in range(1, 11)]
    yAxis = ['A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J']
    for y in range(len(yAxis)):
        for x in range(len(xAxis)):
            if (y % 2) == 0:
                if (x % 2) != 0:
                    tempShot = (f'{yAxis[y]}:{xAxis[x]}')
                    if offPlayer.huntBoard[tempShot] == '0':
                        keyList.append(tempShot)
            elif (y % 2) != 0:
                if (x % 2) == 0:
                    tempShot = (f'{yAxis[y]}:{xAxis[x]}')
                    if offPlayer.huntBoard[tempShot] == '0':
                        keyList.append(tempShot)
    if len(keyList) >= 1:
        return random.choice(keyList)
    else:
        tempList = getOffPlayerHuntboardkey('0', offPlayer)
        return random.choice(tempList)


def probDensityComputerShot(offPlayer):
    while True:
        tempSort = sorted(offPlayer.cumlativeProb.items(),
                          key=lambda x: x[1], reverse=True)
        while True:
            for i in tempSort:
                shotValue = offPlayer.huntBoard.get(i[0], " ")
                if shotValue != '0':
                    pass
                else:
                    return i[0]


def huntShotReturn(offPlayer):
    xAxis = [i for i in range(1, 11)]
    yAxis = ['A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J']
    pos = offPlayer.huntModeStartShot.split(':')
    yShot = pos[0]
    xShot = int(pos[1])
    HuntDict = ['yAxisNorthHunt',
                'xAxisEastHunt',
                'yAxisSouthHunt',
                'xAxisWestHunt']
    for i in HuntDict:
        if i == 'yAxisNorthHunt':
            yShotIndex = (yAxis.index(yShot) - 1)
            if yShotIndex >= 0:
                possShot = (f'{yAxis[yShotIndex]}:{xShot}')
                if offPlayer.huntBoard[possShot] == 'M':
                    pass
                elif offPlayer.huntBoard[possShot] == 'H':
                    checkShotCycle = True
                    tempShotIndex = yShotIndex
                    while checkShotCycle is True:
                        tempShotIndex -= 1
                        if tempShotIndex >= 0:
                            nextPossShot = (f'{yAxis[tempShotIndex]}:{xShot}')
                            if offPlayer.huntBoard[nextPossShot] == 'M':
                                checkShotCycle = False
                            elif offPlayer.huntBoard[nextPossShot] == 'H':
                                continue
                            else:
                                return nextPossShot
                        else:
                            checkShotCycle = False
                else:
                    return possShot

        elif i == 'xAxisEastHunt':
            xShotIndex = (xAxis.index(xShot) + 1)
            if xShotIndex <= 9:
                possShot = (f'{yShot}:{xAxis[xShotIndex]}')
                if offPlayer.huntBoard[possShot] == 'M':
                    pass

                elif offPlayer.huntBoard[possShot] == 'H':
                    checkShotCycle = True
                    tempShotIndex = xShotIndex
                    while checkShotCycle is True:
                        tempShotIndex += 1
                        if tempShotIndex <= 9:
                            nextPossShot = (f'{yShot}:{xAxis[tempShotIndex]}')
                            if offPlayer.huntBoard[nextPossShot] == 'M':
                                checkShotCycle = False
                            elif offPlayer.huntBoard[nextPossShot] == 'H':
                                continue
                            else:
                                return nextPossShot
                        else:
                            checkShotCycle = False
                else:
                    return possShot

        elif i == 'yAxisSouthHunt':
            yShotIndex = (yAxis.index(yShot) + 1)
            if yShotIndex <= 9:
                possShot = (f'{yAxis[yShotIndex]}:{xShot}')
                if offPlayer.huntBoard[possShot] == 'M':
                    pass
                elif offPlayer.huntBoard[possShot] == 'H':
                    checkShotCycle = True
                    tempShotIndex = yShotIndex
                    while checkShotCycle is True:
                        tempShotIndex += 1
                        if tempShotIndex <= 9:
                            nextPossShot = (f'{yAxis[tempShotIndex]}:{xShot}')
                            if offPlayer.huntBoard[nextPossShot] == 'M':
                                checkShotCycle = False
                            elif offPlayer.huntBoard[nextPossShot] == 'H':
                                continue
                            else:
                                return nextPossShot
                        else:
                            checkShotCycle = False
                else:
                    return possShot
        else:
            xShotIndex = (xAxis.index(xShot) - 1)
            if xShotIndex >= 0:
                possShot = (f'{yShot}:{xAxis[xShotIndex]}')
                if offPlayer.huntBoard[possShot] == 'M':
                    pass

                elif offPlayer.huntBoard[possShot] == 'H':
                    checkShotCycle = True
                    tempShotIndex = xShotIndex
                    while checkShotCycle is True:
                        tempShotIndex -= 1
                        if tempShotIndex >= 0:
                            nextPossShot = (f'{yShot}:{xAxis[tempShotIndex]}')
                            if offPlayer.huntBoard[nextPossShot] == 'M':
                                checkShotCycle = False
                            elif offPlayer.huntBoard[nextPossShot] == 'H':
                                continue
                            else:
                                return nextPossShot
                        else:
                            checkShotCycle = False
                else:
                    return possShot


def getOffPlayerHuntboardkey(val, offPlayer):
    tempList = []
    for key, value in offPlayer.huntBoard.items():
        if val == value:
            tempList.append(key)
    return tempList



from .gameFunctions import randomPlayer, fullTurn
from .computerPlays import computerShot
from .playerPlays import playerShot


def printTurnResult(turnResult):
    if turnResult.get('shotResult') == 'M':
        print(f'{turnResult.get("offPlayer")} fired at '
              f'{turnResult.get("shot")} and missed.')
    elif turnResult.get('shotResult') == 'H':
        if turnResult.get('shipSunk') is None:
            print(f'{turnResult.get("offPlayer")} fired at'
                  f'{turnResult.get("shot")} and hit '
                  f'{turnResult.get("defPlayer")}\'s ship.')
        else:
            print(f'{turnResult.get("offPlayer")} fired at '
                  f'{turnResult.get("shot")} and sunk '
                  f'{turnResult.get("defPlayer")}\'s '
                  f'{turnResult.get("shipSunk")}.')


def ComputerGameFunction(player1, player2):
    gameLog = []
    players = randomPlayer(player1, player2)
    offPlayer = players[0]
    defPlayer = players[1]
    gameInProgress = True
    while gameInProgress is True:
        shot = computerShot(offPlayer)
        offPlayer.lastShot = shot
        turnResult = fullTurn(shot, offPlayer, defPlayer)
        gamestatus = turnResult.get('gameStatus', 'in-progress')
        shotResult = turnResult.get('shotResult')
        offPlayer.lastShotResult = shotResult
        if gamestatus == 'over':
            gameLog.append(turnResult)
            gameInProgress = False
        else:
            shotResult = turnResult.get('shotResult', 'Error')
            offPlayer.huntBoard[shot] = shotResult
            gameLog.append(turnResult)
            tempOffPlayer = offPlayer
            offPlayer = defPlayer
            defPlayer = tempOffPlayer

    return gameLog


def OnePlayerGameFunction(player1, player2):
    gameLog = []
    players = randomPlayer(player1, player2)
    offPlayer = players[0]
    defPlayer = players[1]
    gameInProgress = True
    while gameInProgress is True:
        if offPlayer.computer is True:
            shot = computerShot(offPlayer)
        else:
            shot = playerShot(offPlayer)

        offPlayer.lastShot = shot
        turnResult = fullTurn(shot, offPlayer, defPlayer)
        gamestatus = turnResult.get('gameStatus', 'in-progress')
        shotResult = turnResult.get('shotResult')
        offPlayer.lastShotResult = shotResult
        if gamestatus == 'over':
            gameLog.append(turnResult)
            gameInProgress = False
        else:
            shotResult = turnResult.get('shotResult', 'Error')
            offPlayer.huntBoard[shot] = shotResult
            gameLog.append(turnResult)
            printTurnResult(turnResult)
            tempOffPlayer = offPlayer
            offPlayer = defPlayer
            defPlayer = tempOffPlayer



from .players import Player
from .gameBoards import print_hunt_board
from .autoShipPlacement import placeAllships, placementOptions


def playerAutoPlacementValidate(prompt):
    while True:
        try:
            promptValue = int(input(prompt))
        except ValueError:
            print('Sorry, please enter either a 1 for computer '
                  'placement, and or 2 to place your own ships.')
            continue
        if promptValue == 1:
            return True
        elif promptValue == 2:
            return False
        else:
            print('Sorry, please enter either a 1 for computer '
                  'placement, or 2 to place your own ships.')
            continue


def validateLocationInput(prompt):
    xAxis = [i for i in range(1, 11)]
    yAxis = ['A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J']
    while True:
        promptValue = input(prompt)
        pos = promptValue.split(':')
        if len(pos) != 2:
            print('Either the location you provided is not in the '
                  'correct format or you are not using the correct range')
            continue
        try:
            xValue = int(pos[1])
        except ValueError:
            print('Either the location you provided is not in the '
                  'correct format or you are not using the correct range')
            continue

        if pos[0] not in yAxis:
            print('Either the location you provided is not in the '
                  'correct format or you are not using the correct range')
            continue
        elif xValue not in xAxis:
            print('Either the location you provided is not in the '
                  'correct format or you are not using the correct range')
            continue
        else:
            break
    return promptValue


def locationCollisionCheck(location, player):
    if location in player.masterPlacement:
        return True
    else:
        return False


def fullLocationInputValidation(player, playerShip):
    while True:
        tempLocation = validateLocationInput('Please enter the starting '
                                             'location for your '
                                             f'{playerShip.shipName}.  '
                                             'Use a letter between A and J '
                                             'and a number between 1 and 10. '
                                             'Use the example as the '
                                             'format A:9: ')
        locationCollisionCheckResults = locationCollisionCheck(tempLocation,
                                                               player)
        if locationCollisionCheckResults is True:
            print(f'{tempLocation} is currently in use, please select '
                  'another location')
            continue
        else:
            return tempLocation


def optionChoiceValidate(prompt, possEndLocs):
    while True:
        try:
            promptValue = int(input(prompt))
        except ValueError:
            print('Sorry, please enter one of the option numbers')
            continue
        options = [(x+1) for x in range(len(possEndLocs))]
        if promptValue in options:
            return possEndLocs[(promptValue-1)]
        else:
            print('Sorry, please enter one of the option numbers')
            continue


def chooseShipPlacement(player, locationOptions, shipType, validatedLocation):
    player.updateMasterBoard()
    tempBoard = player.masterBoard
    possEndLocs = []
    for i in range(len(locationOptions)):
        if locationOptions[i][0] == validatedLocation:
            possEndLocs.append(locationOptions[i][-1])
        else:
            possEndLocs.append(locationOptions[i][0])
        for j in locationOptions[i]:
            tempBoard[j] = str(i+1)
    tempBoard[validatedLocation] = shipType
    print_hunt_board(tempBoard)
    for i in range(len(possEndLocs)):
        print(f'Option {(i+ 1)} endpoint is {possEndLocs[i]}')
    endLocation = optionChoiceValidate('Type the option number for '
                                       'the endpoint placement: ', possEndLocs)
    return endLocation


def carrierOptions(player, playerShip):
    player.updateMasterBoard()
    while True:
        print_hunt_board(player.masterBoard)
        player.removeCarrier()
        validatedLocation = fullLocationInputValidation(player, playerShip)
        locationOptions = placementOptions(validatedLocation,
                                           player.carrier.size,
                                           player)
        if len(locationOptions) >= 1:
            break
        else:
            print('No valid ship locations found, please select a '
                  'new start position')
            continue
    endLocation = chooseShipPlacement(player,
                                      locationOptions,
                                      player.carrier.shipAbbr,
                                      validatedLocation)
    for i in locationOptions:
        if endLocation in i:
            player.placeCarrier(i)


def battleshipOptions(player, playerShip):
    player.updateMasterBoard()
    player.removeBattleship()
    while True:
        print_hunt_board(player.masterBoard)
        validatedLocation = fullLocationInputValidation(player, playerShip)
        locationOptions = placementOptions(validatedLocation,
                                           player.battleship.size,
                                           player)
        if len(locationOptions) >= 1:
            break
        else:
            print('No valid ship locations found, please select a '
                  'new start position')
            continue
    endLocation = chooseShipPlacement(player,
                                      locationOptions,
                                      player.battleship.shipAbbr,
                                      validatedLocation)
    for i in locationOptions:
        if endLocation in i:
            player.placeBattleship(i)


def cruiserOptions(player, playerShip):
    player.updateMasterBoard()
    player.removeCruiser()
    while True:
        print_hunt_board(player.masterBoard)
        validatedLocation = fullLocationInputValidation(player, playerShip)
        locationOptions = placementOptions(validatedLocation,
                                           player.cruiser.size,
                                           player)
        if len(locationOptions) >= 1:
            break
        else:
            print('No valid ship locations found, please select a '
                  'new start position')
            continue
    endLocation = chooseShipPlacement(player,
                                      locationOptions,
                                      player.cruiser.shipAbbr,
                                      validatedLocation)
    for i in locationOptions:
        if endLocation in i:
            player.placeCruiser(i)


def submarineOptions(player, playerShip):
    player.updateMasterBoard()
    player.removeSubmarine()
    while True:
        print_hunt_board(player.masterBoard)
        validatedLocation = fullLocationInputValidation(player, playerShip)
        locationOptions = placementOptions(validatedLocation,
                                           player.submarine.size,
                                           player)
        if len(locationOptions) >= 1:
            break
        else:
            print('No valid ship locations found, please select a '
                  'new start position')
            continue
    endLocation = chooseShipPlacement(player,
                                      locationOptions,
                                      player.submarine.shipAbbr,
                                      validatedLocation)
    for i in locationOptions:
        if endLocation in i:
            player.placeSubmarine(i)


def destroyerOptions(player, playerShip):
    player.updateMasterBoard()
    player.removeDestroyer()
    while True:
        print_hunt_board(player.masterBoard)
        validatedLocation = fullLocationInputValidation(player, playerShip)
        locationOptions = placementOptions(validatedLocation,
                                           player.destroyer.size,
                                           player)
        if len(locationOptions) >= 1:
            break
        else:
            print('No valid ship locations found, please select a '
                  'new start position')
            continue
    endLocation = chooseShipPlacement(player,
                                      locationOptions,
                                      player.destroyer.shipAbbr,
                                      validatedLocation)
    for i in locationOptions:
        if endLocation in i:
            player.placeDestroyer(i)


def playerAllPlacement(player):
    carrierOptions(player, player.carrier)
    battleshipOptions(player, player.battleship)
    cruiserOptions(player, player.cruiser)
    submarineOptions(player, player.submarine)
    destroyerOptions(player, player.destroyer)


def boardCheckValidate(prompt):
    while True:
        promptValue = input(prompt)
        if promptValue not in ['Y', 'N']:
            print('Please type Y or N')
            continue
        else:
            if promptValue == 'Y':
                return True
            else:
                return False


def shipChoiceValidate(player, prompt):
    while True:
        try:
            promptValue = int(input(prompt))
        except ValueError:
            print('Sorry your responce was not understood, please '
                  'enter 1, 2, 3, 4, 5, 6')
            continue
        if promptValue == 1:
            carrierOptions(player, player.carrier)
            return True
        elif promptValue == 2:
            battleshipOptions(player, player.battleship)
            return True
        elif promptValue == 3:
            cruiserOptions(player, player.cruiser)
            return True
        elif promptValue == 4:
            submarineOptions(player, player.submarine)
            return True
        elif promptValue == 5:
            destroyerOptions(player, player.destroyer)
            return True
        elif promptValue == 6:
            return False
        else:
            print('Sorry your responce was not understood, please '
                  'enter 1, 2, 3, 4, 5, 6')
            continue


def finalPlacementCheck(tempPlayer, player):
    while True:
        tempPlayer.updateMasterBoard()
        print_hunt_board(tempPlayer.masterBoard)
        boardResponce = boardCheckValidate('Would you like to make changes '
                                           'to your ship locations? Press Y '
                                           'for yes and N for No\n')
        if boardResponce is True:
            shipChoiceResult = shipChoiceValidate(tempPlayer, 'Which ship '
                                                  'would you like to replace '
                                                  '\nType 1 for Carrier'
                                                  '\nType 2 for Battleship'
                                                  '\nType 3 for Cruiser'
                                                  '\nType 4 for Submarine'
                                                  '\nType 5 for Destroyer'
                                                  '\nType 6 if you do not '
                                                  'want to make any changes'
                                                  '\n')
            if shipChoiceResult is True:
                continue
            else:
                break
        else:
            break
    player.placeCarrier(tempPlayer.carrier.placement)
    player.placeBattleship(tempPlayer.battleship.placement)
    player.placeCruiser(tempPlayer.cruiser.placement)
    player.placeSubmarine(tempPlayer.submarine.placement)
    player.placeDestroyer(tempPlayer.destroyer.placement)


def playerPlacement(player):
    autoShipPlacement = playerAutoPlacementValidate('Press 1 if you would like'
                                                    ' the computer to select '
                                                    'your ship locations.'
                                                    ' \nPress 2 if you want to'
                                                    ' place your own ships\n')
    # Create a tempPlayer object to store temp ship placements
    tempPlayer = Player()
    if autoShipPlacement is True:
        placeAllships(tempPlayer)
    else:
        playerAllPlacement(tempPlayer)
    finalPlacementCheck(tempPlayer, player)



from .gameBoards import print_hunt_board
from .gameBoards import print_prob_board
from .playerShipPlacement import boardCheckValidate, validateLocationInput

def printProbBoard(offPlayer):
    offPlayer.updateCumlativeProb()
    print_prob_board(offPlayer.cumlativeProb)


def fullShotValidation(offPlayer):
    while True:
        shot = validateLocationInput('Please enter your shot location '
                              'Use a letter between A and J '
                              'and a number between 1 and 10. '
                              'Use the example as the format A:9: ')
        pshot = checkHuntBoardShot(offPlayer, shot)
        if pshot is False:
            print(f'{shot} has already been fired at, please select a new location')
            continue
        else:
            return shot


def checkHuntBoardShot(offPlayer, shot):
        if offPlayer.huntBoard[shot] != ' ':
            return False
        else:
            return True


def playerShot(offPlayer):
    print_hunt_board(offPlayer.huntBoard)
    probBoardResults = boardCheckValidate('Would you like to print the probability matrix? (Y/N)\n')
    if probBoardResults is True:
        printProbBoard(offPlayer)
    shot = fullShotValidation(offPlayer)     


import random
from .gameBoards import generateBoard


def northPlaceCheck(shipSize, yIndexStart, xStart, yAxis):
    northShipPlace = []
    tempLastIndex = (yIndexStart + 1) - shipSize
    if tempLastIndex >= 0 and tempLastIndex <= 10:
        for i in range(tempLastIndex, yIndexStart + 1):
            northShipPlace.append(f'{yAxis[i]}:{xStart}')
        return northShipPlace
    else:
        return False


def southPlaceCheck(shipSize, yIndexStart, xStart, yAxis):
    southShipPlace = []
    tempLastIndex = yIndexStart + shipSize
    if tempLastIndex >= 0 and tempLastIndex <= 10:
        for i in range(yIndexStart, tempLastIndex):
            southShipPlace.append(f'{yAxis[i]}:{xStart}')
        return southShipPlace
    else:
        return False


def westPlaceCheck(shipSize, xIndexStart, yStart, xAxis):
    westShipPlace = []
    tempLastIndex = (xIndexStart + 1) - shipSize
    if tempLastIndex >= 0 and tempLastIndex <= 10:
        for i in range(tempLastIndex, xIndexStart + 1):
            westShipPlace.append(f'{yStart}:{xAxis[i]}')
        return westShipPlace

    else:
        return False


def eastPlaceCheck(shipSize, xIndexStart, yStart, xAxis):
    eastShipPlace = []
    tempLastIndex = xIndexStart + shipSize
    if tempLastIndex >= 0 and tempLastIndex <= 10:
        for i in range(xIndexStart, tempLastIndex):
            eastShipPlace.append(f'{yStart}:{xAxis[i]}')
        return eastShipPlace
    else:
        return False


def placementOptions(loc1, ship, player):
    possLocList = []
    xAxis = [i for i in range(1, 11)]
    yAxis = ['A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J']
    pos = loc1.split(':')
    yStart = pos[0]
    xStart = int(pos[1])
    yIndexStart = (yAxis.index(yStart))
    xIndexStart = xAxis.index(xStart)
    northPlace = northPlaceCheck(ship, yIndexStart, xStart, yAxis)
    if northPlace is not False:
        if any(x in northPlace for x in player.masterPlacement):
            pass
        else:
            possLocList.append(northPlace)
    southPlace = southPlaceCheck(ship, yIndexStart, xStart, yAxis)
    if southPlace is not False:
        if any(x in southPlace for x in player.masterPlacement):
            pass
        else:
            possLocList.append(southPlace)
    westPlace = westPlaceCheck(ship, xIndexStart, yStart, xAxis)
    if westPlace is not False:
        if any(x in westPlace for x in player.masterPlacement):
            pass
        else:
            possLocList.append(westPlace)
    eastPlace = eastPlaceCheck(ship, xIndexStart, yStart, xAxis)
    if eastPlace is not False:
        if any(x in eastPlace for x in player.masterPlacement):
            pass
        else:
            possLocList.append(eastPlace)
    return possLocList


def randomLocation():
    gameBoard = generateBoard()
    gameBoardList = [*gameBoard]
    possibleLocation = random.choice(gameBoardList)
    return possibleLocation


def findCarrierLocation(player):
    while True:
        while True:
            possibleLocation = randomLocation()
            if possibleLocation in player.masterPlacement:
                continue
            else:
                break
        locationOptions = placementOptions(possibleLocation,
                                           player.carrier.size, player)
        if len(locationOptions) >= 1:
            return random.choice(locationOptions)
        else:
            continue


def findBattleshipLocation(player):
    while True:
        while True:
            possibleLocation = randomLocation()
            if possibleLocation in player.masterPlacement:
                continue
            else:
                break
        locationOptions = placementOptions(possibleLocation,
                                           player.battleship.size, player)
        if len(locationOptions) >= 1:
            return random.choice(locationOptions)
        else:
            continue


def findCruiserLocation(player):
    while True:
        while True:
            possibleLocation = randomLocation()
            if possibleLocation in player.masterPlacement:
                continue
            else:
                break
        locationOptions = placementOptions(possibleLocation,
                                           player.cruiser.size,
                                           player)
        if len(locationOptions) >= 1:
            return random.choice(locationOptions)
        else:
            continue


def findSubmarineLocation(player):
    while True:
        while True:
            possibleLocation = randomLocation()
            if possibleLocation in player.masterPlacement:
                continue
            else:
                break
        locationOptions = placementOptions(possibleLocation,
                                           player.submarine.size,
                                           player)
        if len(locationOptions) >= 1:
            return random.choice(locationOptions)
        else:
            continue


def findDestroyerLocation(player):
    while True:
        while True:
            possibleLocation = randomLocation()
            if possibleLocation in player.masterPlacement:
                continue
            else:
                break
        locationOptions = placementOptions(possibleLocation,
                                           player.destroyer.size,
                                           player)
        if len(locationOptions) >= 1:
            return random.choice(locationOptions)
        else:
            continue


def placeAllships(player):
    player.placeCarrier(findCarrierLocation(player))
    player.placeBattleship(findBattleshipLocation(player))
    player.placeCruiser(findCruiserLocation(player))
    player.placeSubmarine(findSubmarineLocation(player))
    player.placeDestroyer(findDestroyerLocation(player))



from setuptools import setup, find_packages

VERSION = '0.0.4'
DESCRIPTION = 'A basic CLI program to play battleship against the computer'

setup(
    name='cliBattleship',
    version=VERSION,
    author='Tyson Van Patten',
    author_email='tyson@descriptdata.com',
    url='https://github.com/tvanpat/cli-battleship',
    description=DESCRIPTION,
    packages=find_packages(),
    install_requires=[]
)
